Package: postNet
Type: Package
Title: Post-transcriptional network modeling
Description: A tool that enables in silico identification, integration, and modeling of mRNA features that influence post-transcriptional regulation of gene expression at a transcriptome-wide scale.
Version: 1.1.0
Authors@R: c(person("Krzysztof", "Szkop", email = "krzysztof.szkop@gmail.com", role = c("aut","cre")),
    person("Kathleen", "Watt", email = "watt.kathleen@gmail.com", role = "aut"),
    person("Ola", "Larsson", email = "ola.larsson@ki.se", role = "aut"))
License: MIT + file LICENSE
Encoding: UTF-8
LazyLoad: true
Depends: R (>= 4.5.0),
Imports: dplyr, plyr, Biostrings, data.table, gridExtra, seqinr,
        R.utils, reshape2, vioplot, stringr, plotrix, gplots, ggplot2,
        ggrepel, anota2seq, memes, GenomicRanges, IRanges, WriteXLS,
        randomForest, igraph, Boruta, ROCR, caret, msigdb,
        ExperimentHub, AnnotationHub, GSEABase, fgsea, org.Hs.eg.db,
        org.Mm.eg.db, RColorBrewer, httr2, rvest, umap, clusterProfiler
        (>= 4.18.4), gage, withr, grDevices, graphics, methods, stats,
        utils, tools, BiocFileCache, curl
LinkingTo: Rcpp, BH
Suggests: knitr, rmarkdown, BiocStyle, pdftools, magick, testthat (>=
        3.0.0)
biocViews: GeneExpression, GeneRegulation, Transcriptomics, RiboSeq,
        RNASeq, Sequencing, Annotation, Network, FeatureExtraction
VignetteBuilder: knitr
URL: https://github.com/kszkop/postNet
BugReports: https://github.com/kszkop/postNet/issues
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/postNet
git_branch: devel
git_last_commit: 6b1e4b6
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:26:17 UTC; biocbuild
Author: Krzysztof Szkop [aut, cre],
  Kathleen Watt [aut],
  Ola Larsson [aut]
Maintainer: Krzysztof Szkop <krzysztof.szkop@gmail.com>
