export(signCalc)
export(slopeFilt)
export(lengthAnalysis)
export(miRNAanalysis)
export(rfPred)
export(gseaAnalysis)
export(gseaPlot)
export(codonCalc)
export(signaturesHeatmap)
export(gageAnalysis)
export(contentAnalysis)
export(motifAnalysis)
export(goAnalysis)
export(plotFeaturesMap)
export(uorfAnalysis)
export(contentMotifs)
export(foldingEnergyAnalysis)
export(goDotplot)
export(plotSignatures)
export(plotSignatures_ads)
export(featureIntegration)
export(postNetStart)
export(codonUsage)
export(get_signatures)
export(ptn_codonSelection)
export(ptn_sequences)
export(ptn_id)
export(ptn_geneID)
export(ptn_dataIn)
export(ptn_geneList)
export(ptn_background)
export(ptn_effect)
export(ptn_colours)
export(ptn_species)
export(ptn_version)
export(ptn_selection)
export(ptn_motifSelection)
export(ptn_motifGeneList)
export(ptn_codonAnalysis)
export(ptn_features)
export(ptn_miRNA_analysis)
export(ptn_miRNA_to_gene)
export(ptn_GO)
export(ptn_GSEA)
export(ptn_GAGE)
export(ptn_check_models)
export(ptn_model)
export(ptn_selectedFeatures)
export(ptn_networkGraph)

useDynLib(postNet, .registration=TRUE)

import(dplyr)
importFrom(plyr,ldply)
importFrom(Biostrings, DNAString)
importFrom(dplyr,"%>%")
importFrom(data.table,":=")
importFrom("grDevices", "adjustcolor", "dev.off", "pdf")
importFrom("graphics", "abline", "axis", "boxplot", "dotchart",
               "layout", "legend", "lines", "mtext", "par", "plot.new",
               "plot.window", "points", "rect", "text")
importFrom("methods", "new", "slot", "slot<-", "slotNames")
importFrom("stats", "IQR", "anova", "as.formula", "chisq.test",
               "coefficients", "complete.cases", "cor", "cor.test", "ecdf",
               "end", "fisher.test", "frequency", "lm", "median",
               "na.omit", "p.adjust", "predict", "qchisq", "quantile",
               "reorder", "setNames", "smooth.spline", "start",
               "wilcox.test")
importFrom("utils", "combn", "download.file", "read.delim",
               "read.table", "tail", "write.table", "data")
importFrom("GenomicRanges","GRanges")
importFrom("IRanges","IRanges")
