# postNet
### Post-transcriptional Network Modeling 
<br>

  Post-transcriptional mechanisms play a central role in the regulation of gene expression, with protein levels partly determined by various features within target mRNAs. Emerging evidence indicates that most individual mRNAs contain multiple regulatory elements. This underscores the need for efficient bioinformatic tools that can capture and integrate multiple mRNA features to assess both their independent and combined impact on the proteome. Here, we present postNet, a tool that enables in silico identification, integration, and modeling of mRNA features that influence post-transcriptional regulation of gene expression at a transcriptome-wide scale. Although geared towards studies of post-transcriptional regulation, postNet is highly customizable and can, in principle, be applied in a variety of other contexts to explain changes in a continuous numeric variable between two or more conditions/groups. This vignette provides details regarding the use of postNet, and demonstrates typical workflows and results interpretation. 
  
------------------------------------------------------------------------

## Data for Post-transcriptional Network Modeling (postNet)

postNetData provides curated datasets required by the postNet framework for post-transcriptional network modelling. The package includes reference transcript annotations. It is designed as a companion data enabling users to run postNet workflows without additional data preparation.

Refseq versions:

Human: ver_40.202408

Mouse: ver_27.202402

CCDS Release 25 (2022)

------------------------------------------------------------------------

## Citing postNet

Please cite the following article when using __postNet__:

------------------------------------------------------------------------

## License

[![Packagist](https://img.shields.io/packagist/l/doctrine/orm.svg?maxAge=2592000?style=flat)](https://opensource.org/licenses/MIT)

------------------------------------------------------------------------

## Contacts

krzysztof.szkop@gmail.com

