utils::globalVariables(c(
    ".",
    "AA",
    "AACountPerGene",
    "AAcodon",
    "Description",
    "UMAP1",
    "UMAP2",
    "all_codons",
    "category",
    "codon",
    "colVecColour",
    "geneID",
    "group",
    "is_dataframe",
    "lenTmp",
    "log10fdr",
    "regulation",
    "x",
    "y"
))
