\name{postNetVignette}
\alias{postNetVignette}
\docType{data}
\title{An example dataset for demonstrating the workflow of a postNet ananlysis}

\description{This object contains example data to run a postNet analysis, including lists of post-transcriptionally regulated and background genes, and log2 fold changes in translation efficiency between two treatment conditions.
}

\usage{data(postNetVignette, package='postNet')}

\format{
  The format is:
List of 5
 $ geneList  :List of 2
  ..$ translationUp  : chr [1:1445] "DDX6" "CLK1" "DUSP1" "TXNIP" ...
  ..$ translationDown: chr [1:1378] "PPDPF" "NCKAP1" "AP2M1" "ABHD12" ...
 $ background: chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
 $ effect    : Named num [1:9555] -2.095 0.275 -0.57 -0.608 0.576 ...
  ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
 $ features  :List of 18
  ..$ UTR5_length                    : Named num [1:9555] 8.34 7.18 7.23 7.71 4.91 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ UTR3_length                    : Named num [1:9555] 9.59 4.86 11.96 9.46 11.07 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ UTR5_C                         : Named num [1:9555] 28.1 32.4 54.7 30.1 36.7 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ UTR5_A                         : Named num [1:9555] 18.5 14.5 6 21.1 16.7 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ UTR5_T                         : Named num [1:9555] 19.4 22.1 14 14.8 10 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ UTR3_G                         : Named num [1:9555] 36.8 10.3 26.6 16.6 23.4 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ UTR3_C                         : Named num [1:9555] 26.3 13.8 25.9 17 17.9 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ UTR3_A                         : Named num [1:9555] 20.4 27.6 20 34.1 28.1 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ UTR3_T                         : Named num [1:9555] 16.5 48.3 27.4 32.3 30.5 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ Up                             : Named num [1:9555] 0.00567 0.05068 0.04141 0.11059 0.13968 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ Down                           : Named num [1:9555] 0.2125 0.1326 0.1871 0.0541 0.0762 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ uORFs_ATG_strong               : Named num [1:9555] 2 0 0 0 0 0 0 0 0 0 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ UTR5_SCSCGS                    : Named num [1:9555] 0 1 5 1 0 3 0 0 0 1 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ Gandin_etal_2016_mTOR_transUp  : Named num [1:9555] 1 0 0 0 0 0 0 0 0 0 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ Gandin_etal_2016_mTOR_transDown: Named num [1:9555] 0 0 0 0 0 0 0 0 0 0 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ Cockman_etal_2020_classicTOP   : Named num [1:9555] 0 0 0 0 0 0 0 0 0 0 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ Guan_etal_2017_Tg1_transUp     : Named num [1:9555] 0 0 0 0 0 0 0 0 0 0 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
  ..$ Guan_etal_2017_Tg1_transDown   : Named num [1:9555] 0 0 0 0 0 0 0 0 0 0 ...
  .. ..- attr(*, "names")= chr [1:9555] "A4GALT" "AAAS" "AACS" "AADAT" ...
 $ ads_data  :List of 3
  ..$ phenoVec: chr [1:8] "Control" "Control" "Control" "Control" ...
  ..$ dataP   : num [1:100, 1:8] 6.15 7.85 8.62 6.15 5.8 ...
  .. ..- attr(*, "dimnames")=List of 2
  .. .. ..$ : chr [1:100] "ACOX1" "ACTR2" "AGRN" "APOBEC3C" ...
  .. .. ..$ : chr [1:8] "riboSeq_Control_rep1" "riboSeq_Control_rep2" "riboSeq_Control_rep3" "riboSeq_Control_rep4" ...
  ..$ dataT   : num [1:100, 1:8] 5.77 8.23 9.39 5.34 6.4 ...
  .. ..- attr(*, "dimnames")=List of 2
  .. .. ..$ : chr [1:100] "ACOX1" "ACTR2" "AGRN" "APOBEC3C" ...
  .. .. ..$ : chr [1:8] "rnaSeq_Control_rep1" "rnaSeq_Control_rep2" "rnaSeq_Control_rep3" "rnaSeq_Control_rep4" ...
}

\details{The example data provided with the postNet package is taken from a study of the effect of osmotic stress on mRNA translation. Immortalized human corneal epithelial cells (10.014O pRSV-T) were cultured under osmotic stress (500 mOsm, NaCl) for 1h, along with controls.The provided gene lists are detrived from an anota2seq analysis of ribosome profiling and RNA-seq data to identify genes that were translationally activated or suppressed under osmotic stress. 

A set of representative mRNA features was also enumerated in these translationally regulated genes, along with signatures of genes for which translation is know to be regulated downstream of mTOR and the integrated stress response (ISR). This set of features is used to illustrate the feature integration modelling and UMAP visualizations.  
}

\source{
Krokowski D, Jobava R, Szkop KJ, Chen CW, Fu X, Venus S, Guan BJ, Wu J, Gao Z, Banaszuk W, Tchorzewski M, Mu T, Ropelewski P, Merrick WC, Mao Y, Sevval AI, Miranda H, Qian SB, Manifava M, Ktistakis NT, Vourekas A, Jankowsky E, Topisirovic I, Larsson O, Hatzoglou M. Stress-induced perturbations in intracellular amino acids reprogram mRNA translation in osmoadaptation independently of the ISR. Cell Rep. 2022 Jul 19;40(3):111092. doi: 10.1016/j.celrep.2022.111092. PMID: 35858571; PMCID: PMC9491157.

Data available at GEO: GSE200097. 
}

\examples{
postNetVignette <- data(postNetVignette, package='postNet')
str(postNetVignette)
}
\keyword{datasets}
