\name{postNetExample}
\alias{postNetExample}
\docType{data}
\title{An example dataset for demonstrating the usage of postNet functions}

\description{This object contains example data for testing postNet functions, including lists of post-transcriptionally regulated genes, background gene sets, and log_2 fold changes in translation efficiency between two treatment conditions.
}

\usage{data(postNetExample, package='postNet')}

\format{
  The format is:
List of 6
 $ geneList  :List of 2
  ..$ translationUp  : chr [1:10] "OCIAD1" "MSH2" "SCFD1" "INO80C" ...
  ..$ translationDown: chr [1:10] "XKR8" "ANPEP" "AGPAT2" "UBN2" ...
 $ background: chr [1:100] "SMARCD2" "ZNF239" "MTIF3" "ADGRE2" ...
 $ effect    : Named num [1:100] -0.849 -2.506 -2.615 1.676 0.315 ...
  ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
 $ ptn       :Formal class 'postNetData' [package "postNet"] with 8 slots
  .. ..@ species  : chr "human"
  .. ..@ version  : chr "ver_40.202408"
  .. ..@ selection: chr "random"
  .. ..@ seed     : num 123
  .. ..@ annot    :Formal class 'postNetAnnot' [package "postNet"] with 4 slots
  .. .. .. ..@ UTR5:Formal class 'postNetRegion' [package "postNet"] with 3 slots
  .. .. .. .. .. ..@ id       : chr [1:100] "NM_152916" "NM_001135189" "NM_001012727" "NM_020133" ...
  .. .. .. .. .. ..@ geneID   : chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  .. .. .. .. .. ..@ sequences: chr [1:100] "ACAGCTGCCCAGCCTGCGGAGACGGGACAGCCCTGTCCCACTCACTCTTTCCCCTGCTGCTCCTGCCGGCAGCTCAGCTGGAACC" "CCCCTCAGGAGAAGTCGGGAAGGTGGCGGCGGCGGCGGCGGTTGTCCCGGCTGTGCCGGTTGGTGTGGCCCGTCAGCCCGCGTACCACAGCGCCCGGGCCGCGTCGAGCCC"| __truncated__ "AGCCCCGCCGCCCTCGCAATAAGGGGCCTGAGCGCGCGGGGGAGAAGCGGGAGCGGGAGCGGGAGCGAGCTGGCGGCGCCGTCGGGCGCCGGGCCGGGCC" "GCTTTTTCTTTCCAGTGTTGGCTGACTTACAGCTCTTATAAACTAGTGGCAATTTCTGAACCCAGCCGGCTCCATCTCAGCTTCTGGTTTCTAAGTCCATGTGCCAAAGGC"| __truncated__ ...
  .. .. .. ..@ CDS :Formal class 'postNetRegion' [package "postNet"] with 3 slots
  .. .. .. .. .. ..@ id       : chr [1:100] "NM_152916" "NM_001135189" "NM_001012727" "NM_020133" ...
  .. .. .. .. .. ..@ geneID   : chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  .. .. .. .. .. ..@ sequences: chr [1:100] "ATGGGAGGCCGCGTCTTTCTCGTCTTTCTCGCATTCTGTGTCTGGCTGACTCTGCCGGGAGCTGAAACCCAGGACTCCAGGGGCTGTGCCCGGTGGTGCCCTCAGGACTCC"| __truncated__ "ATGGCGGCCAGCGCGAAGCGGAAGCAGGAGGAGAAGCACCTGAAGATGCTGCGGGACATGACCGGCCTCCCGCACAACCGAAAGTGCTTCGACTGCGACCAGCGCGGCCCC"| __truncated__ "ATGGAGCTGTGGCCGTGTCTGGCCGCGGCGCTGCTGTTGCTGCTGCTGCTGGTGCAGCTGAGCCGCGCGGCCGAGTTCTACGCCAAGGTCGCCCTGTACTGCGCGCTGTGC"| __truncated__ "ATGGACCTCGCGGGACTGCTGAAGTCTCAGTTCCTGTGCCACCTGGTCTTCTGCTACGTCTTTATTGCCTCAGGGCTAATCATCAACACCATTCAGCTCTTCACTCTCCTC"| __truncated__ ...
  .. .. .. ..@ UTR3:Formal class 'postNetRegion' [package "postNet"] with 3 slots
  .. .. .. .. .. ..@ id       : chr [1:100] "NM_152916" "NM_001135189" "NM_001012727" "NM_020133" ...
  .. .. .. .. .. ..@ geneID   : chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  .. .. .. .. .. ..@ sequences: chr [1:100] "AAAAATCTTCTGAATAAGATCTTCCCTCTTTGCCCGTGGAAAATCTGAACAATCTTTGAGCCATCTAGAGGGGAAAGAAAAGACTTTGTTCTGTGTGTTTCAAGAAATTCA"| __truncated__ "CCTTATATAGACAATTTACTGGAACGAACTTTTATGTGGTCACATTACATCTCTCCACCTCTTGCACTGTTGTCTTGTTTCACTGATCTTAGCTTTAAACACAAGAGAAGT"| __truncated__ "CCCAGACCACGGCAGGGCATGACCTGGGGAGGGCAGGTGGAAGCCGATGGCTGGAGGATGGGCAGAGGGGACTCCTCCCGGCTTCCAAATACCACTCTGTCCGGCTCCCCC"| __truncated__ "CTCAGGGAGGTGTCACCATCCGAAGGGAACCTTGGGGAACTGGTGGCCTCTGCATATCCTCCTTAGTGGGACACGGTGACAAAGGCTGGGTGAGCCCCTGCTGGGCACGGC"| __truncated__ ...
  .. .. .. ..@ CCDS: NULL
  .. ..@ dataIn   :Formal class 'postNetDataIn' [package "postNet"] with 4 slots
  .. .. .. ..@ background: chr [1:100] "SMARCD2" "ZNF239" "MTIF3" "ADGRE2" ...
  .. .. .. ..@ geneList  :List of 2
  .. .. .. .. ..$ translationUp  : chr [1:10] "OCIAD1" "MSH2" "SCFD1" "INO80C" ...
  .. .. .. .. ..$ translationDown: chr [1:10] "XKR8" "ANPEP" "AGPAT2" "UBN2" ...
  .. .. .. ..@ effect    : Named num [1:100] -0.849 -2.506 -2.615 1.676 0.315 ...
  .. .. .. .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  .. .. .. ..@ colours   : chr [1:2] "#7FB7BE" "#DB7F67"
  .. ..@ features : NULL
  .. ..@ analysis :Formal class 'postNetAnalysis' [package "postNet"] with 7 slots
  .. .. .. ..@ featureIntegration: NULL
  .. .. .. ..@ motifs            : NULL
  .. .. .. ..@ codons            : NULL
  .. .. .. ..@ GO                : NULL
  .. .. .. ..@ GSEA              : NULL
  .. .. .. ..@ GAGE              : NULL
  .. .. .. ..@ miRNA             : NULL
 $ ads_data  :List of 3
  ..$ phenoVec: chr [1:8] "Control" "Control" "Control" "Control" ...
  ..$ dataP   : num [1:100, 1:8] 3.3 6.54 4.69 3.62 5.78 ...
  .. ..- attr(*, "dimnames")=List of 2
  .. .. ..$ : chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  .. .. ..$ : chr [1:8] "riboSeq_Control_rep1" "riboSeq_Control_rep2" "riboSeq_Control_rep3" "riboSeq_Control_rep4" ...
  ..$ dataT   : num [1:100, 1:8] 3.53 6.78 5.64 3.05 4.85 ...
  .. ..- attr(*, "dimnames")=List of 2
  .. .. ..$ : chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  .. .. ..$ : chr [1:8] "rnaSeq_Control_rep1" "rnaSeq_Control_rep2" "rnaSeq_Control_rep3" "rnaSeq_Control_rep4" ...
 $ features  :List of 17
  ..$ UTR5_length     : Named num [1:100] 6.41 8.06 6.64 7.73 5.25 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ CDS_length      : Named num [1:100] 11.18 10.62 9.53 10.15 10.61 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR3_length     : Named num [1:100] 11.91 12.71 9.25 12.68 4.17 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR5_G          : Named num [1:100] 24.7 42.1 49 21.6 44.7 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR5_C          : Named num [1:100] 42.4 39.8 33 24.9 18.4 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR5_A          : Named num [1:100] 15.29 8.27 13 20.19 18.42 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR5_T          : Named num [1:100] 17.65 9.77 5 33.33 18.42 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ CDS_G           : Named num [1:100] 25.6 21.7 31.2 27.1 19.2 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ CDS_C           : Named num [1:100] 30.5 26.1 33.6 27.2 23.1 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ CDS_A           : Named num [1:100] 21.8 27.3 15.5 22.2 23.6 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ CDS_T           : Named num [1:100] 22.1 24.9 19.7 23.6 34.1 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR3_G          : Named num [1:100] 20.7 17.5 34 23.9 22.2 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR3_C          : Named num [1:100] 22.3 15.7 32.5 22.7 11.1 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR3_A          : Named num [1:100] 28.7 30.8 17.7 27.1 44.4 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR3_T          : Named num [1:100] 28.3 36 15.8 26.2 22.2 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ uORFs_ATG_strong: Named num [1:100] 0 0 0 0 0 0 0 0 0 1 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
  ..$ UTR5_SGCSGCS    : Named num [1:100] 0 6 2 0 0 2 0 0 0 1 ...
  .. ..- attr(*, "names")= chr [1:100] "ADGRE2" "AGFG1" "AGPAT2" "AGPAT4" ...
}

\details{The example data provided is a small subset of the data taken from a study of the effect of osmotic stress on mRNA translation. Immortalized human corneal epithelial cells (10.014O pRSV-T) were cultured under osmotic stress (500 mOsm, NaCl) for 1h, along with controls.The provided gene lists are detrived from an anota2seq analysis of ribosome profiling and RNA-seq data to identify genes that were translationally activated or suppressed under osmotic stress. 

A set of representative mRNA features was also enumerated in these translationally regulated genes, along with signatures of genes for which translation is know to be regulated downstream of mTOR and the integrated stress response (ISR). This set of features is used to illustrate the feature integration modelling and UMAP visualizations.  
}

\source{
Krokowski D, Jobava R, Szkop KJ, Chen CW, Fu X, Venus S, Guan BJ, Wu J, Gao Z, Banaszuk W, Tchorzewski M, Mu T, Ropelewski P, Merrick WC, Mao Y, Sevval AI, Miranda H, Qian SB, Manifava M, Ktistakis NT, Vourekas A, Jankowsky E, Topisirovic I, Larsson O, Hatzoglou M. Stress-induced perturbations in intracellular amino acids reprogram mRNA translation in osmoadaptation independently of the ISR. Cell Rep. 2022 Jul 19;40(3):111092. doi: 10.1016/j.celrep.2022.111092. PMID: 35858571; PMCID: PMC9491157.

Data available at GEO: GSE200097. 
}

\examples{
postNetExample <- data(postNetExample, package='postNet')
str(postNetExample)
}
\keyword{datasets}
