% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postNet-package.R
\name{postNet-package}
\alias{postNet}
\alias{postNet-package}
\title{Post-transcriptional network modelling with postNet}

\description{
  Tools for the identification, integration, and modelling of mRNA features
  that influence post-transcriptional regulation of gene expression at a
  transcriptome-wide scale.
}

\details{
  
  For an introduction to the package and example workflows, see the
  package vignette:
    
    \code{vignette("postNet")}
}

\author{
  See \code{packageDescription("postNet")}.
}

\docType{package}

\seealso{
  Useful starting points:
    
    \code{vignette("postNet")}
  
  Package overview and workflow examples are provided in the vignette.
}

\keyword{GeneExpression}
\keyword{GeneRegulation}
\keyword{Transcriptomics}
\keyword{RiboSeq}
\keyword{RNASeq}
\keyword{Sequencing}
\keyword{Annotation}
\keyword{Network}
\keyword{FeatureExtraction}
